﻿using Newtonsoft.Json;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations.Schema;

namespace Kehyeedra3.Services.Models
{
    public enum FishSize
    {
        Small = 0,
        Medium = 1,
        Large = 2
    }
    public enum FishRarity
    {
        Common = 1, 
        Uncommon = 2, 
        Rare = 3, 
        Legendary = 4, 
        Unreasonable
    }
    public enum FishTier
    {
        T1 = 1, 
        T2 = 2, 
        T3 = 3, 
        T4 = 4 
    }
    public class Fish
    {
        public FishSpecies Id;
        //public byte Ego;
        //public byte Superego;
        public string Name = "Missing Name";
        public string Emote = "<:missingLeg:682586847830081551>";
        public string Texture = "missingtexture.png";
        public FishRarity Rarity;
        public FishTier Tier;
    }
    public class Fishing
    {
        public ulong Id { get; set; } = 0; // this is userid incase you get alzheimers you stupid baby waa waa
        public ulong LastFish { get; set; } = 0;
        public ulong Xp { get; set; } = 50;
        public ulong FXp { get; set; } = 0;
        public ulong TXp { get; set; } = 0;
        public ulong Lvl { get; set; } = 0;
        public byte RodOwned { get; set; } = 0;
        public byte RodUsed { get; set; } = 0;
        public int Prestige { get; set; } = 0;

        [Column(TypeName="LONGTEXT")]
        public string Inventory { get; set; } = "{}";

        public Dictionary<FishSpecies, int[]> GetInventory()
        {
            return JsonConvert.DeserializeObject<Dictionary<FishSpecies, int[]>>(Inventory);
        }

        public void SetInventory(Dictionary<FishSpecies, int[]> inv)
        {
            Dictionary<int, int[]> temp = new            Dictionary<int, int[]>();
            foreach(var entry in inv){
                temp.Add((int)entry.Key, entry.Value);
            }
            Inventory = JsonConvert.SerializeObject(temp);
        }

        public static List<Fish> GetFishList()
        {
            return new List<Fish>
            {
                new Fish()
                {
                    Id = FishSpecies.Wakasagihime,
                    Name = "Freshwater Lady",
                    Emote = "<:wakasagihime:793084118949691433>",
                    Texture = "touhoufish.png",
                    Rarity = FishRarity.Unreasonable,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.LuckyCatfish,
                    Name = "Lucky Catfish",
                    Emote = "<a:catfishleft:682655661422542888><a:catfishright:682655661481525284>",
                    Texture = "catfish.gif",
                    Rarity = FishRarity.Legendary,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Doomfish,
                    Name = "Doomfish",
                    Emote = "<a:doomfish:704407234574155834>",
                    Texture = "doomfish.gif",
                    Rarity = FishRarity.Rare,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Clownfish,
                    Name = "Clownfish",
                    Emote = "<a:clownfishleft:715846565972934718><a:clownfishright:715846565704761424>",
                    Texture = "clownfish.gif",
                    Rarity = FishRarity.Rare,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Teracrab,
                    Name = "Teracrab",
                    Emote = "<a:teracrableft:710925664089538691><a:teracrabright:710925663439421512>",
                    Texture = "teracrab.gif",
                    Rarity = FishRarity.Rare,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Blobfish,
                    Name = "Blobfish",
                    Emote = "<a:blobfishleft:704386995996065885><a:blobfishright:704386996369358888>",
                    Texture = "blobfish.gif",
                    Rarity = FishRarity.Rare,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Psychedelica,
                    Name = "Psychedelica",
                    Emote = "<a:psychedelicaleft:704406253966721135><a:psychedelicaright:704406252125421698>",
                    Texture = "psychedelica.gif",
                    Rarity = FishRarity.Rare,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Gigacrab,
                    Name = "Gigacrab",
                    Emote = "<:gigacrableft:715626112327221348><:gigacrabright:715626112537067580>",
                    Texture = "gigacrab.png",
                    Rarity = FishRarity.Uncommon,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Dopefish,
                    Name = "Dopefish",
                    Emote = "<:dopefishleft:700422139672658009><:dopefishright:700422139643428895>",
                    Texture = "dopefish.png",
                    Rarity = FishRarity.Uncommon,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Stargazer,
                    Name = "Stargazer",
                    Emote = "<:stargazerleft:700414644774240286><:stargazerright:700413063442202684>",
                    Texture = "stargazer.png",
                    Rarity = FishRarity.Uncommon,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Isopod,
                    Name = "Isopod",
                    Emote = "<:isopodleft:700397032271249428><:isopodright:700397031922991206>",
                    Texture = "isopod.png",
                    Rarity = FishRarity.Uncommon,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Sheephead,
                    Name = "Sheephead",
                    Emote = "<:sheepheadleft:710894977944649728><:sheepheadright:710894979467444284>",
                    Texture = "sheephead.png",
                    Rarity = FishRarity.Uncommon,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Cod,
                    Name = "Cod",
                    Emote = "<:codleft:695304941715062887><:codright:695304941949943808>",
                    Texture = "cod.png",
                    Rarity = FishRarity.Common,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Salmon,
                    Name = "Salmon",
                    Emote = "<:salmonleft:698167269359878236><:salmonright:698167269167202324>",
                    Texture = "salmon.png",
                    Rarity = FishRarity.Common,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Shrimp,
                    Name = "Shrimp",
                    Emote =  "<:shrimpleft:715074288788570112><:shrimpright:715074288863936552>",
                    Texture = "shrimp.png",
                    Rarity = FishRarity.Common,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Crayfish,
                    Name = "Crayfish",
                    Emote =  "<:crayfishleft:715638929885495377><:crayfishright:715638929843421284>",
                    Texture = "crayfish.png",
                    Rarity = FishRarity.Common,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Betta,
                    Name = "Betta",
                    Emote = "<:bettaleft:698179217107714088><:bettaright:698179216868638851>",
                    Texture = "betta.png",
                    Rarity = FishRarity.Common,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Pufferfish,
                    Name = "Pufferfish",
                    Emote = "<:pufferfishleft:715075414179184691><:pufferfishright:715075414116007937>",
                    Texture = "pufferfish.png",
                    Rarity = FishRarity.Common,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Carp,
                    Name = "Carp",
                    Emote = "<:koicarpleft:698204388556275752><:koicarpright:698204386421374986>",
                    Texture = "carp.png",
                    Rarity = FishRarity.Common,
                    Tier = FishTier.T1
                },
                new Fish()
                {
                    Id = FishSpecies.Megacrab,
                    Name = "Hermit Crab",
                    Emote = "<:hermitcrableft:715071501996392519><:hermitcrabright:715071501971488808>",
                    Texture = "hermitcrab.png",
                    Rarity = FishRarity.Common,
                    Tier = FishTier.T1
                },

                new Fish() //// Tier 2
                {
                    Id = FishSpecies.Hypnofish, // replace with shrimp waifu fish when art done
                    Name = "Hypnofish",
                    Emote = "<:paska:786244602440450109><:hypnoosi:786244623478947841>",
                    Texture = "hypnofish.png",
                    Rarity = FishRarity.Legendary,
                    Tier = FishTier.T2
                },
                new Fish()
                {
                    Id = FishSpecies.Swolefish,
                    Name = "Pumped Up Swolefish",
                    //Texture = "",
                    Rarity = FishRarity.Rare,
                    Tier = FishTier.T2
                },
                new Fish()
                {
                    Id = FishSpecies.Dogfish,
                    Name = "Dogfish",

                    Rarity = FishRarity.Rare,
                    Tier = FishTier.T2
                },
                new Fish()
                {
                    Id = FishSpecies.Wololo,
                    Name = "Wololo",

                    Rarity = FishRarity.Rare,
                    Tier = FishTier.T2
                },
                new Fish()
                {
                    Id = FishSpecies.Gunfish,
                    Name = "Gunfish",
                    Emote = "<:gunfishleft:793492588799590460><:gunfishright:793492625277714442>",
                    Texture = "gunfish.png",
                    Rarity = FishRarity.Uncommon,
                    Tier = FishTier.T2
                },
                new Fish()
                {
                    Id = FishSpecies.Dingus,
                    Name = "Dingus",

                    Rarity = FishRarity.Uncommon,
                    Tier = FishTier.T2
                },
                new Fish()
                {
                    Id = FishSpecies.Finnfish,
                    Name = "Fi'ish",

                    Rarity = FishRarity.Uncommon,
                    Tier = FishTier.T2
                },
                new Fish()
                {
                    Id = FishSpecies.Rockfish,
                    Name = "Rockfish",

                    Rarity = FishRarity.Common,
                    Tier = FishTier.T2
                },
                new Fish()
                {
                    Id = FishSpecies.Fishlet,
                    Name = "Fishlet",

                    Rarity = FishRarity.Common,
                    Tier = FishTier.T2
                },
                new Fish()
                {
                    Id = FishSpecies.Cavefish,
                    Name = "Cavefish",

                    Rarity = FishRarity.Common,
                    Tier = FishTier.T2
                },
                new Fish()
                {
                    Id = FishSpecies.Neck,
                    Name = "Neck",

                    Rarity = FishRarity.Common,
                    Tier = FishTier.T2
                },

                new Fish() //// Tier 3
                {
                    Id = FishSpecies.Circlefish,
                    Name = "Circle Fish",

                    Rarity = FishRarity.Legendary,
                    Tier = FishTier.T3
                },
                new Fish()
                {
                    Id = FishSpecies.RevCrab,
                    Name = "Revenant Crab",
                    
                    Rarity = FishRarity.Rare,
                    Tier = FishTier.T3
                },
                new Fish()
                {
                    Id = FishSpecies.SpecFlameworm,
                    Name = "Spectral Flameworm",
                    
                    Rarity = FishRarity.Uncommon,
                    Tier = FishTier.T3
                },
                new Fish()
                {
                    Id = FishSpecies.SpecShrimp,
                    Name = "Spectral Shrimp",

                    Rarity = FishRarity.Common,
                    Tier = FishTier.T3
                },
                new Fish()
                {
                    Id = FishSpecies.Ghostie,
                    Name = "Ghostie",

                    Rarity = FishRarity.Common,
                    Tier = FishTier.T3
                },


                new Fish()
                {
                    Id = FishSpecies.T4PH1,
                    Name = "Placeholder",

                    Rarity = FishRarity.Legendary,
                    Tier = FishTier.T4
                },
                new Fish()
                {
                    Id = FishSpecies.T4PH2,
                    Name = "Placeholder",

                    Rarity = FishRarity.Rare,
                    Tier = FishTier.T4
                },
                new Fish()
                {
                    Id = FishSpecies.T4PH3,
                    Name = "Placeholder",

                    Rarity = FishRarity.Uncommon,
                    Tier = FishTier.T4
                },
                new Fish()
                {
                    Id = FishSpecies.T4PH4,
                    Name = "Placeholder",

                    Rarity = FishRarity.Common,
                    Tier = FishTier.T4
                },

           };
        }
    }

    public enum FishSpecies
    {
        //unreasonable
        Wakasagihime = 0,
        //legendary
        LuckyCatfish = 1,
        //rare
        Doomfish = 2,
        Clownfish = 3,
        Teracrab = 4,
        Blobfish = 5,
        Psychedelica = 6,
        //uncommon
        Gigacrab = 7,
        Dopefish = 8,
        Stargazer = 9,
        Isopod = 10,
        Sheephead = 11,
        //common
        Cod = 12,
        Salmon = 13,
        Pufferfish = 14,
        Shrimp = 15,
        Crayfish = 16,
        Betta = 17,
        Carp = 18,
        Megacrab = 19,
        //T2 Legendary
        Hypnofish = 20,
        //T2 Rare
        Swolefish = 21,
        Dogfish = 22,
        Wololo = 23,
        //T2 Uncommon
        Gunfish = 30,
        Dingus = 31,
        Finnfish = 32,
        //T2 Common
        Rockfish = 40,
        Fishlet = 41,
        Cavefish = 42,
        Neck = 43,

        //T3 Legendary
        Circlefish = 50,
        //T3 Rare
        RevCrab = 51,
        //T3 Uncommon
        SpecFlameworm = 60,
        //T3 Common
        SpecShrimp = 70,
        Ghostie = 71,
        //T4
        T4PH1 = 80,
        //r
        T4PH2 = 90,
        //uc
        T4PH3 = 100,
        //c
        T4PH4 = 110,
    }
    public class FishingInventorySlot
    {
        public int Id;
        public int[] Amount;
    }
}
